﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Windows.Input;
using FocusVMLib;
using MvvmFoundation.Wpf;

namespace WpfApplication1
{
    public class Person : ObservableObject, IDataErrorInfo, IFocusMover
    {
        #region Properties

        public string FirstName
        {
            get { return _firstName; }
            set
            {
                _firstName = value;
                base.RaisePropertyChanged("FirstName");
            }
        }

        public string LastName
        {
            get { return _lastName; }
            set
            {
                _lastName = value;
                base.RaisePropertyChanged("LastName");
            }
        }

        public bool? IsLegalAge
        {
            get { return _isLegalAge; }
            set
            {
                _isLegalAge = value;
                base.RaisePropertyChanged("IsLegalAge");
            }
        }

        public ICommand SaveCommand
        {
            get { return new RelayCommand(this.Save); }
        }

        #endregion // Properties

        #region Save

        void Save()
        {
            if (this.HasError)
            {
                // The Error property knows which property is invalid.
                this.RaiseMoveFocus(this.Error);
            }
            else
            {
                // Save to database...
                System.Windows.MessageBox.Show("Saved");
            }
        }

        bool HasError
        {
            get { return _validatedProperties.Any(prop => !String.IsNullOrEmpty(this[prop])); }
        }

        #endregion // Save

        #region IDataErrorInfo Members

        public string Error { get; private set; }

        public string this[string property]
        {
            get
            {
                string msg;
                if (this.IsValid(property, out msg))
                {
                    this.Error = null;
                }
                else
                {
                    // Keep track of the most recently 
                    // validated property that is invalid.
                    this.Error = property;                    
                }
                return msg;
            }
        }

        bool IsValid(string property, out string errorMessage)
        {
            errorMessage = null;
            if (property == "FirstName")
            {
                if (IsNullOrWhitespace(this.FirstName))
                {
                    errorMessage = "First name is missing.";
                }
            }
            else if (property == "LastName")
            {
                if (IsNullOrWhitespace(this.LastName))
                {
                    errorMessage = "Last name is missing.";
                }
            }
            else
            {
                if (!this.IsLegalAge.HasValue)
                {
                    errorMessage = "Legal age not verified.";
                }
            }
            return errorMessage == null;
        }

        static bool IsNullOrWhitespace(string s)
        {
            return String.IsNullOrEmpty(s) || s.Trim().Length == 0;
        }

        #endregion

        #region IFocusMover Members

        public event EventHandler<MoveFocusEventArgs> MoveFocus;

        void RaiseMoveFocus(string focusedProperty)
        {
            var handler = this.MoveFocus;
            if (handler != null)
            {
                var args = new MoveFocusEventArgs(focusedProperty);
                handler(this, args);
            }
        }

        #endregion

        #region Fields

        // In a real app, these strings could be located via reflection over
        // custom attributes, instead of being hard-coded like this...
        static string[] _validatedProperties = new string[]
        {
            "FirstName",
            "LastName",
            "IsLegalAge"
        };

        string _firstName, _lastName;
        bool? _isLegalAge = null;

        #endregion // Fields
    }
}